YayMounts = LibStub("AceAddon-3.0"):NewAddon("YayMounts","AceConsole-3.0", "AceEvent-3.0")
AceGUI = LibStub("AceGUI-3.0")

local options = {
	name="YayMounts",
	handler = YayMounts,
	type = "group",
	args = {
		enable = {
			type = "execute",
			name = YayMountsLocale.Misc.optionsGUIButton,
			func = "Frames",
		},
	}
}

local version = "1.1.5"
local t = { }
local playerCombat, onGroundMount, speedAspect, mountUsed
local _, class = UnitClass("player")

local defaults = { 
	char = { 
		settings = {
			["DB"] = false, --database
			["ER"] = true, --emptyrandom: use random mount if none selected
			["FFP"] = true, --Druid: Use flight form with priority
			["TTT"] = false, --Travel to Travel, no human form. Mostly for pvp.
			["MTFF"] = false, --Druid: When flying mounted shift directly to flight form
			["CFI"] = true, --Druid: Cat Form Indoors (with 2/2 Feral Swiftness)
			["FBP"] = false, --Prioritize flying broom, even over Flight form
			["POR"] = false, --Pack or cheetah
			["Remount"] = false, --Remount from ground->flying with 1 button
			["HM"] = false, -- Has Mount
			["NDWF"] = false, -- No Dismount while Flying. Safe!
		},
		misc = {
			["OGM"] = false, --on ground mount
			["CV"] = 1.0, --Current version
		},		
		mounts = {
			['*'] = {
			},
		},
	}
}

function YayMounts:OnInitialize() 
	self.db = LibStub("AceDB-3.0"):New("YayMountsDB", defaults)
	local ymOptions = LibStub("AceConfigRegistry-3.0")
	ymOptions:RegisterOptionsTable("YayMounts",options)
	self.ymOpFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("YayMounts","YayMounts")
	self:RegisterChatCommand("ym", "ChatCommand")
	self:RegisterChatCommand("yaymounts", "ChatCommand")
end

function YayMounts:ChatCommand(input) --Chat command function
	if input == "reload" or input == "rl" then
		YayMounts:SortMounts()
		self:Print(YayMountsLocale.Misc.mountsReloaded)
	else
		YayMounts:Frames()
	end
end

function YayMounts:OnEnable()-- addon enabled
	if not self.db.char.settings["DB"] or self.db.char.misc["CV"] ~= version then
		YayMounts:SortMounts()
		self.db.char.misc["CV"] = version
	end
	YayMounts:MapOverrides()
 	
	self:RegisterEvent("UPDATE_BINDINGS", "MapOverrides")
	self:RegisterEvent("COMPANION_LEARNED", "SortMounts")
	if class ==  "DRUID" or class == "SHAMAN" or class == "HUNTER" or class == "PRIEST" or class == "MAGE" then
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "EventHandler")
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "EventHandler")
	end
end

--------------------------------------------------------------
--database handling and creation functions
--------------------------------------------------------------
local function IsUsableS(spellID)
	local bool = IsUsableSpell(YayMounts:SpellToName(spellID))
	return bool
end

local function IsUsableM(MountSID)
	for q=1, GetNumCompanions("MOUNT"), 1 do
		local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if S == MountSID then
			return true
		end
	end
	return false
end

function YayMounts:SpellToName(spell) --converts spell ID to Name
	local name = GetSpellInfo(spell)
	return name
end

function YayMounts:ItemToName(item)
	local name = GetItemInfo(item)
	return name
end

function YayMounts:GetMounts(info, key)
	if info == "mounts" then
		return self.db.char.mounts[key][4]
	elseif info == "settings" then
		return self.db.char.settings[key]
	end
end

function YayMounts:SetMounts(info, key, state)
	if info == "mounts" then
		self.db.char.mounts[key][4] = state
	elseif info == "settings" then
		if key == "Remount" then
			self.db.char.misc["OGM"] = state
			self.db.char.settings[key] = state
		else
			self.db.char.settings[key] = state
		end
	end
end

function YayMounts:ClearMounts(mountType)-- clear all mounts
	for q=1,GetNumCompanions("MOUNT"), 1 do
		local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][3] == mountType then
			self.db.char.mounts[S][4] = false
			t[S]:SetValue(false)
		end
	end
end

function YayMounts:SelectAllMounts(mountType)-- select all mounts
	for q=1,GetNumCompanions("MOUNT"), 1 do
		local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][3] == mountType then
			self.db.char.mounts[S][4] = true
			t[S]:SetValue(true)
		end
	end
end

function YayMounts:BuildList(mountType)--Builds and draws the checkboxes for the opened tab
	for q=1, GetNumCompanions("MOUNT"), 1 do
	local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][3] == mountType then
			t[S] = AceGUI:Create("CheckBox")
			t[S]:SetImage(self.db.char.mounts[S][2])
			t[S]:SetLabel(self.db.char.mounts[S][1])
			t[S]:SetValue(YayMounts:GetMounts("mounts", S))
			t[S]:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("mounts", S, t[S]:GetValue()) end)
			YayMountsContent:AddChild(t[S])
		end
	end
	
	if mountType == "ground" and IsUsableS(87840) then
		t[87840] = AceGUI:Create("CheckBox")
		t[87840]:SetImage("Interface\\Icons\\ability_mount_blackdirewolf")
		t[87840]:SetLabel("Running Wild")
		t[87840]:SetValue(YayMounts:GetMounts("mounts", 87840))
		t[87840]:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("mounts", 87840, t[87840]:GetValue()) end)
		YayMountsContent:AddChild(t[87840])
	end
end

function YayMounts:SortMounts()--Rebuilds the database. Activates on load TODO CODE ON EVENT LEARN MOUNT
	for q=1, GetNumCompanions("MOUNT"), 1 do
		local _,_,S,I,O = GetCompanionInfo("MOUNT", q)
		local N = YayMounts:SpellToName(S)
		local mountType = nil
		for k,v in pairs(YayMountsType["Mounts"]) do
			if S == k then
				mountType = v
			end
		end
		if mountType == nil and S ~= 75207 then
			self:Print(YayMountsLocale.String.NewMountFound..""..N)
		else
			if self.db.char.mounts[S][4] == nil then
				self.db.char.mounts[S] = {N, I, mountType, true}
			elseif self.db.char.mounts[S][4] == true then
				self.db.char.mounts[S] = {N, I, mountType, true}
			else
				self.db.char.mounts[S] = {N, I, mountType, false}
			end
			self.db.char.settings["HM"] = true
		end
	end
	
	if IsUsableS(87840) then
		if self.db.char.mounts[87840] then
			if self.db.char.mounts[87840][4] == nil then
				self.db.char.mounts[87840] = {"Running Wild", "Interface\\Icons\\ability_mount_blackdirewolf", "ground", false}
			elseif self.db.char.mounts[87840][4] == true then
				self.db.char.mounts[87840] = {"Running Wild", "Interface\\Icons\\ability_mount_blackdirewolf", "ground", true}
			else
				self.db.char.mounts[87840] = {"Running Wild", "Interface\\Icons\\ability_mount_blackdirewolf", "ground", false}
			end
			
		else
			self.db.char.mounts[87840] = {"Running Wild", "Interface\\Icons\\ability_mount_blackdirewolf", "ground", true}
		end
		
		self.db.char.settings["HM"] = true
	end
	self.db.char.settings["DB"] = true
end

--------------------------------------------------------------
--GUI functions for drawing frames
--------------------------------------------------------------
local function DrawGroup1a(container) --Ground Mounts, Tab1
	container:SetLayout("Flow")
	
	local selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("ground") end)
	container:AddChild(selectallbutton)
		
	local clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("ground") end)
	container:AddChild(clearallbutton)
	
	local scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	local scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
	
	YayMountsContent = AceGUI:Create("InlineGroup")
	YayMountsContent:SetFullWidth(true)
 	YayMountsContent:SetLayout("Flow")

	scrollframe:AddChild(YayMountsContent)
	YayMounts:BuildList("ground")
end

local function DrawGroup2a(container) --Flying Mounts, Tab2
	container:SetLayout("Flow")
	
	local selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("fly") end)
	container:AddChild(selectallbutton)
		
	local clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("fly") end)
	container:AddChild(clearallbutton)
	
	local scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	local scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
	
	YayMountsContent = AceGUI:Create("InlineGroup")
	YayMountsContent:SetFullWidth(true)
 	YayMountsContent:SetLayout("Flow")

	scrollframe:AddChild(YayMountsContent)
	YayMounts:BuildList("fly")
end

local function DrawGroup3a(container) --Scaling Mounts, Tab3
	container:SetLayout("Flow")
	
	local selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("scaling") end)
	container:AddChild(selectallbutton)
		
	local clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("scaling") end)
	container:AddChild(clearallbutton)
		
	local scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)
		
	local scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
		
	YayMountsContent = AceGUI:Create("InlineGroup")
	YayMountsContent:SetFullWidth(true)
 	YayMountsContent:SetLayout("Flow")
		
	scrollframe:AddChild(YayMountsContent)
	YayMounts:BuildList("scaling")
end

local function DrawGroup4a(container) --Special Mounts, Tab4
	container:SetLayout("Flow")
	
	local selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("scaling") end)
	container:AddChild(selectallbutton)
		
	local clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("scaling") end)
	container:AddChild(clearallbutton)
		
	local scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)
		
	local scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
		
	YayMountsContent = AceGUI:Create("InlineGroup")
	YayMountsContent:SetFullWidth(true)
 	YayMountsContent:SetLayout("Flow")
		
	scrollframe:AddChild(YayMountsContent)
	YayMounts:BuildList("special")
end

local function SelectGroupMounts(container, event, group)
	container:ReleaseChildren()
	if group == "tab1" then
		DrawGroup1a(container)
	elseif group == "tab2" then
		DrawGroup2a(container)
	elseif group == "tab3" then
		DrawGroup3a(container)
	elseif group == "tab4" then
		DrawGroup4a(container)
	end
end

local function DrawGroup1(container)-- function that draws the widgets for the first tab MOUNTS
	container:SetLayout("Fill")

	local tab  = AceGUI:Create("TabGroup")
	tab:SetLayout("Flow")
	tab:SetTabs({{text="Ground", value="tab1"}, {text="Flying", value="tab2"}, {text="Scaling", value="tab3"}, {text="Special", value="tab4"}})
	tab:SetCallback("OnGroupSelected", SelectGroupMounts)
	tab:SelectTab("tab1")
	container:AddChild(tab)
end


local function DrawGroup3(container)-- function that draws the widgets for the third tab OPTIONS
	container:SetLayout("Flow")
	
	yayMountsBindingButton = AceGUI:Create("Keybinding")
	yayMountsBindingButton:SetLabel(BINDING_NAME_YAYMOUNTBINDING)
	yayMountsBindingButton:SetKey(GetBindingKey("YAYMOUNTBINDING"))
	yayMountsBindingButton:SetCallback("OnKeyChanged", function() YayMounts:Keybinding(yayMountsBindingButton:GetKey(), 1) end)
	container:AddChild(yayMountsBindingButton)
	
	yayMountsBindingButton2 = AceGUI:Create("Keybinding")
	yayMountsBindingButton2:SetLabel(BINDING_NAME_YAYMOUNTBINDING2)
	yayMountsBindingButton2:SetKey(GetBindingKey("YAYMOUNTBINDING2"))
	yayMountsBindingButton2:SetCallback("OnKeyChanged", function() YayMounts:Keybinding(yayMountsBindingButton2:GetKey(), 2); end)
	container:AddChild(yayMountsBindingButton2)

	local nDWF = AceGUI:Create("CheckBox")
	nDWF:SetLabel(YayMountsLocale.String.NDWF)
	nDWF:SetValue(YayMounts:GetMounts("settings", "NDWF"))
	nDWF:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "NDWF", nDWF:GetValue()) end)
	nDWF:SetFullWidth(true)
	container:AddChild(nDWF)
	
	local emptyRandom = AceGUI:Create("CheckBox")
	emptyRandom:SetLabel(YayMountsLocale.String.ER)
	emptyRandom:SetValue(YayMounts:GetMounts("settings", "ER"))
	emptyRandom:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "ER", emptyRandom:GetValue()) end)
	emptyRandom:SetFullWidth(true)
	container:AddChild(emptyRandom)
	
	local remount = AceGUI:Create("CheckBox")
	remount:SetLabel(YayMountsLocale.String.Remount)
	remount:SetValue(YayMounts:GetMounts("settings", "Remount"))
	remount:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "Remount", remount:GetValue()) end)
	remount:SetFullWidth(true)
	container:AddChild(remount)
	
	if GetItemCount(33176, false) > 0 then
		local flyingBroomPriority = AceGUI:Create("CheckBox")
		if class == "DRUID" then
			flyingBroomPriority:SetLabel(YayMountsLocale.String.FBP)
		else
			flyingBroomPriority:SetLabel(YayMountsLocale.String.FBPD)
		end
		flyingBroomPriority:SetValue(YayMounts:GetMounts("settings", "FBP"))
		flyingBroomPriority:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "FBP", flyingBroomPriority:GetValue()) end)
		flyingBroomPriority:SetFullWidth(true)
		container:AddChild(flyingBroomPriority)
	end
	
	if class == "DRUID" then
		local flightFormPriority = AceGUI:Create("CheckBox")
		flightFormPriority:SetLabel(YayMountsLocale.String.FFP)
		flightFormPriority:SetValue(YayMounts:GetMounts("settings", "FFP"))
		flightFormPriority:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "FFP", flightFormPriority:GetValue()) end)
		flightFormPriority:SetFullWidth(true)
		container:AddChild(flightFormPriority)
		
		local travelToTravel = AceGUI:Create("CheckBox")
		travelToTravel:SetLabel(YayMountsLocale.String.TTT)
		travelToTravel:SetValue(YayMounts:GetMounts("settings", "TTT"))
		travelToTravel:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "TTT", travelToTravel:GetValue()) end)
		travelToTravel:SetFullWidth(true)
		container:AddChild(travelToTravel)
		
		local mountToFlightForm = AceGUI:Create("CheckBox")
		mountToFlightForm:SetLabel(YayMountsLocale.String.MTFF)
		mountToFlightForm:SetValue(YayMounts:GetMounts("settings", "MTFF"))
		mountToFlightForm:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "MTFF", mountToFlightForm:GetValue()) end)
		mountToFlightForm:SetFullWidth(true)
		container:AddChild(mountToFlightForm)
		
		local catFormIndoors = AceGUI:Create("CheckBox")
		catFormIndoors:SetLabel(YayMountsLocale.String.CFI)
		catFormIndoors:SetValue(YayMounts:GetMounts("settings", "CFI"))
		catFormIndoors:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "CFI", catFormIndoors:GetValue()) end)
		catFormIndoors:SetFullWidth(true)
		container:AddChild(catFormIndoors)
	end
	
	if class == "HUNTER" then
		local packOrCheetah = AceGUI:Create("CheckBox")
		packOrCheetah:SetLabel(YayMountsLocale.String.POR)
		packOrCheetah:SetValue(YayMounts:GetMounts("settings", "POR"))
		packOrCheetah:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "POR", packOrCheetah:GetValue()) end)
		packOrCheetah:SetFullWidth(true)
		container:AddChild(packOrCheetah)
	end
end

local function SelectGroup(container, event, group)-- Callback function for OnGroupSelected
	container:ReleaseChildren()
	if group == "tab1" then
		DrawGroup1(container)
--~ 	elseif group == "tab2" then
--~ 		DrawGroup2(container)
	elseif group == "tab3" then
		DrawGroup3(container)
	end
end

function YayMounts:Frames()-- Basic frame
	if not yayFrame then
		yayFrame = AceGUI:Create("Frame")
		yayFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget); yayFrame = nil end)
		yayFrame:SetTitle("Yay Mounts")
		yayFrame:SetStatusText("Version "..version..", by Cyrae - Windrunner US")
		yayFrame:SetLayout("Fill")
		yayFrame:SetWidth(720)
		yayFrame:SetHeight(490)
	
		local tab  = AceGUI:Create("TabGroup")
		tab:SetLayout("Flow")
		tab:SetTabs({{text="Mounts", value="tab1"}, {text="Options", value="tab3"}})
		tab:SetCallback("OnGroupSelected", SelectGroup)
		tab:SelectTab("tab1")
		yayFrame:AddChild(tab)
	end
end

--------------------------------------------------------------
--Mounting and button click functions
--------------------------------------------------------------
function YayMounts:Mounting(flyable, zone)
	if self.db.char.settings["HM"] then --fix this for running wild
		local e = {}
		local i = 1
		local mountType
		
		if flyable then
			mountType = "fly"
		else
			mountType = "ground"
		end
		
		--write code to prefer scaling mounts over ground mounts when underwater, unless in vashjir?
		for k,v in pairs(self.db.char.mounts) do
			if (v[3] == mountType or v[3] == "scaling") and v[4] then
				e[i] = k
				i = i + 1
			end
		end
		
		e = YayMounts:Filter(e, zone)
		if #e == 0 and self.db.char.settings["ER"] then
			i = 1
			for k,v in pairs(self.db.char.mounts) do
				if (v[3] == mountType or v[3] == "scaling") then
					e[i] = k
					i = i + 1
				end
			end
			e = YayMounts:Filter(e, zone)
		elseif #e == 0 then
			self:Print(YayMountsLocale.String.NoMounts)
			return nil
		end
		
		if flyable and self.db.char.settings["Remount"] then
			self.db.char.misc["OGM"] = false
		elseif IsOutdoors() and not flyable then
			self.db.char.misc["OGM"] = true
		end
		mountUsed = YayMounts:SpellToName(e[math.random(#e)])
		return mountUsed
	end
end

function YayMounts:PreClick()
local _,_,_,_, ghostWolf = GetTalentInfo(2, 6)
local _,_,_,_, feralSwiftness = GetTalentInfo(2, 1)
local zone = GetRealZoneText()
--local flyable = IsFlyableArea()
local broom = false
local loanedMount = nil

	if zone == YayMountsLocale.Zone.Wintergrasp and GetWintergraspWaitTime() ~= nil then --Check if in wintergrasp and no battle
		flyable = true
	end
	
	if IsUsableSpell(61294) == 1 then
		flyable = true
	else
		flyable = false
	end
	
	loanedMount = YayMounts:CheckMountTraining(zone, loanedMount) --check if we have cold weather flying, a loaned mount, and/or flight master's license

	if ghostwolf == 2 and IsUsableS(2645) then --check if we have instant ghost wolf
		ghostWolf = true
	end
	
	if GetUnitSpeed("player") ~= 0 or IsFalling() or IsFlying() or self.db.char.settings["FBP"] == true then -- check if we can use flying broom
		if GetItemCount(33176, false) > 0 and not playerCombat then
			broom = true
		end
	end
		
	--Smart mounting stuff

	if IsMounted() and IsFlying() then
		if class == "DRUID" and self.db.char.settings["MTFF"] and IsFlying() and GetUnitSpeed("player") ~= 0 then -- Druid: Flying mount to flight form
			YayMounts:Druids(flyable, zone)
			
		elseif self.db.char.settings["NDWF"] then
			YayMounts:SetMacroButton()
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
	
	elseif IsMounted() then
		if self.db.char.settings["Remount"] then  --Stuff for direct ground->flying mounting
			if self.db.char.misc["OGM"] and flyable then
				Dismount()
				self.db.char.misc["OGM"] = false
				if class == "DRUID" then
					YayMounts:Druids(flyable, zone)
				else
					YayMounts:PreClick()
				end
			else
				Dismount()
				YayMounts:SetMacroButton()
			end
			
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
		
	elseif CanExitVehicle() then
		YayMounts:SetMacroButton()
		VehicleExit()
		
	elseif YayMounts:PriestOrMage() then
		return
		
	elseif flyable and IsOutdoors() then
		if class == "SHAMAN" and ghostWolf and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
			
		elseif IsSwimming() then
			YayMounts:Swimming(flyable, zone, broom) --DO SWIMMING
			
		elseif class == "HUNTER" and GetUnitSpeed("player") ~= 0 then
			YayMounts:Hunters()
			
		elseif broom then --flying broom
			YayMounts:SetMacroButton(YayMounts:ItemToName(33176))
			
		elseif class == "DRUID" then
			YayMounts:Druids(flyable, zone)
			
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone))
		end
		
	elseif not flyable and IsOutdoors() and zone ~= YayMountsLocale.Zone.VortexPinnacle then
		if zone == YayMountsLocale.Zone.Oculus and YayMounts:Filter(nil, zone) then
			return
			
		elseif class == "SHAMAN" and ghostWolf and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
			
		elseif IsSwimming() then
			YayMounts:Swimming(flyable, zone, broom)
			
		elseif loanedMount ~= nil and not playerCombat and GetUnitSpeed("player") == 0 then
			YayMounts:SetMacroButton(YayMounts:ItemToName(loanedMount))
			
		elseif class == "DRUID" then
			YayMounts:Druids(flyable, zone)
			
		elseif class == "HUNTER" and GetUnitSpeed("player") ~= 0 then
			YayMounts:Hunters()
			
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone))
		end
		
	elseif not IsOutdoors() or zone == YayMountsLocale.Zone.VortexPinnacle then
		if IsSwimming() then
			YayMounts:Swimming(flyable, zone, broom)
			
		elseif class == "DRUID" and feralSwiftness == 2 and self.db.char.settings["CFI"] then
			YayMounts:SetMacroButton(YayMounts:SpellToName(768))
			
		elseif class == "SHAMAN" and IsUsableS(2645) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
			
		elseif class == "HUNTER" then
			YayMounts:Hunters()
		end
		
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone))
	end
end

function YayMounts:PriestOrMage()	
	if (IsFalling() or (playerCombat and GetUnitSpeed("player") ~= 0)) and class == "PRIEST" and IsUsableS(1706) then --IsUsableSpell checks for reagents/glyphs
		YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(1706)) --Levitate
		return true
		
	elseif (IsFalling() or (playerCombat and GetUnitSpeed("player") ~= 0)) and class == "MAGE" and IsUsableS(130) then
		YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(130)) --Slow fall
		return true
		
	else
		return false
	end
end

function YayMounts:Hunters()
local cheetah = UnitBuff("player", YayMounts:SpellToName(5118))
local pack = UnitBuff("player", YayMounts:SpellToName(13159))
	if playerCombat then
		if IsUsableS(13159) and self.db.char.settings["POR"] and not pack then
			YayMounts:SetMacroButton(""..YayMounts:SpellToName(13159)..", "..YayMounts:SpellToName(13165).."", 3) --Aspect of the Pack
			
		elseif IsUsableS(5118) and not cheetah then
			YayMounts:SetMacroButton(""..YayMounts:SpellToName(5118)..", "..YayMounts:SpellToName(13165).."", 3) --Aspect of the Cheetah
			
		elseif IsUsableS(13159) and self.db.char.settings["POR"] then
			YayMounts:SetMacroButton(""..YayMounts:SpellToName(13165)..", "..YayMounts:SpellToName(13159).."", 3) --Aspect of the Pack
		elseif IsUsableS(5118) then
			YayMounts:SetMacroButton(""..YayMounts:SpellToName(13165)..", "..YayMounts:SpellToName(5118).."", 3) --Aspect of the Cheetah
		end
	else
		if IsUsableS(13159) and self.db.char.settings["POR"] and not pack then
			YayMounts:SetMacroButton(YayMounts:SpellToName(13159)) --Aspect of the Pack
			
		elseif IsUsableS(5118) and not self.db.char.settings["POR"] and not cheetah then
			YayMounts:SetMacroButton(YayMounts:SpellToName(5118)) --Aspect of the Cheetah
			
		elseif cheetah or pack then
			YayMounts:SetMacroButton(YayMounts:SpellToName(13165)) --Aspect of the Hawk
		end
	end
end

function YayMounts:Druids(flyable, zone)
	local flightForm = 5
	if IsUsableS(24858) or IsUsableS(33891) then
		flightform = 6
	end
	
	if (flyable and not playerCombat) then
		if IsSwimming() then
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone))
		elseif (GetUnitSpeed("player") == 0 and self.db.char.settings["FFP"]) or IsFalling() or IsFlying() or GetUnitSpeed("player") ~= 0 or GetShapeshiftForm() == flightForm then
			if IsUsableS(40120) then --Swift Flight Form
				YayMounts:SetMacroButton(YayMounts:SpellToName(40120))
			elseif IsUsableS(33943) then --Flight Form
				YayMounts:SetMacroButton(YayMounts:SpellToName(33943))
			end
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone))
		end
		
	elseif IsUsableS(783) and (playerCombat or GetUnitSpeed("player") ~= 0) then --Travel Form
		YayMounts:SetMacroButton(YayMounts:SpellToName(783))
		
	elseif not playerCombat then
		YayMounts:SetMacroButton(YayMounts:Mounting(false, zone))	
	end
end

function YayMounts:Swimming(flyable, zone, broom)
	local timer, _,_,scale,_,_ = GetMirrorTimerInfo(2)
	local name, _, _, _, _, _, _, _, _, _, _ = UnitBuff("player", YayMountsLocale.Misc.WaterBreathing)
	if timer == "UNKNOWN" or scale >= 0 then
		scale = 1
	end
	
	if flyable and not playerCombat then
		if class == "DRUID" and IsUsableS(1066) and GetUnitSpeed("player") ~= 0 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(1066)) --Aquatic Form
		elseif class == "DRUID" then
			YayMounts:Druids(flyable, zone)
		elseif broom then
			YayMounts:SetMacroButton(YayMounts:ItemToName(33176)) --Flying Broom
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(true, zone))
		end
		
	elseif IsOutdoors() and not playerCombat then
		if class == "DRUID" and IsUsableS(1066) and GetUnitSpeed("player") ~= 0 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(1066)) --Aquatic Form
		else
			if zone == YayMountsLocale.Zone.Vashjir1 or zone == YayMountsLocale.Zone.Vashjir2 or zone == YayMountsLocale.Zone.Vashjir3 and IsUsableM(75207) then --Abyssal Seahorse
				YayMounts:SetMacroButton(YayMounts:SpellToName(75207)) --Abyssal Seahorse
			elseif IsUsableM(64731) then
				YayMounts:SetMacroButton(YayMounts:SpellToName(64731)) --Sea Turtle
			else
				YayMounts:SetMacroButton(YayMounts:Mounting(false, zone))
			end
		--elseif class == "HUNTER" then
			--YayMounts:Hunters()
		end
		
	elseif not playerCombat then
		if class == "DRUID" and IsUsableS(1066) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(1066)) -- Aquatic Form
		elseif class == "SHAMAN" and IsUsableS(2645) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		elseif class == "HUNTER" then
			YayMounts:Hunters()
		end
	end
end

function YayMounts:CheckMountTraining(zone, loanedMount)
	if zone == YayMountsLocale.Zone.StormPeaks or zone == YayMountsLocale.Zone.Icecrown then
		if GetItemCount(44221, false) > 0 then
			return 44221
		elseif GetItemCount(44229, false) > 0 then
			return 44229
		else
			return nil
		end
	end
	return loanedMount, flyable
end

function YayMounts:SetMacroButton(mount, button)
	if class == "DRUID" and GetShapeshiftForm() == 5 and IsUsableS(24858) then -- remove this if blizzard fixes moonkins not able to mount
		YayMountsButton:SetAttribute("macrotext", "/cancelform")
	elseif mount and button == 2 then
		YayMountsButton2:SetAttribute("macrotext", "/use "..YayMounts:Mounting(false, GetRealZoneText()))
	elseif mount and button == 3 then --hunter castsequence macro
		YayMountsButton:SetAttribute("macrotext", "/castsequence "..mount)
	elseif mount then
		YayMountsButton:SetAttribute("macrotext", "/use "..mount)
	else
		YayMountsButton:SetAttribute("macrotext", nil)
	end
end

function YayMounts:MapOverrides()
	if not InCombatLockdown() then
		ClearOverrideBindings(YayMountsButton)
		local key1, key2 = GetBindingKey("YAYMOUNTBINDING")
		if key1 then
			SetOverrideBindingClick(YayMountsButton, true, key1, YayMountsButton:GetName())
		end
		if key2 then
			SetOverrideBindingClick(YayMountsButton, true, key2, YayMountsButton:GetName())
		end
		
		key1, key2 = GetBindingKey("YAYMOUNTBINDING2")
		if key1 then
			SetOverrideBindingClick(YayMountsButton2, true, key1, YayMountsButton2:GetName())
		end
		if key2 then
			SetOverrideBindingClick(YayMountsButton2, true, key2, YayMountsButton2:GetName())
		end
	end
end

function YayMounts:Filter(e, zone)
	if zone ~= YayMountsLocale.Zone.AQ40 and e ~= nil then
		for k,v in pairs(e) do
			if v == YayMountsType["Ahn'Qiraj"][v] then
				tremove(e, k)
			end
		end
		return e
	
	elseif zone == YayMountsLocale.Zone.Oculus then
		if GetItemCount(37859) == 1 then
			YayMounts:SetMacroButton(YayMounts:ItemToName(37859))
			return true
		elseif GetItemCount(37860) == 1 then
			YayMounts:SetMacroButton(YayMounts:ItemToName(37860))
			return true
		elseif GetItemCount(37815) == 1 then
			YayMounts:SetMacroButton(YayMounts:ItemToName(37815))
			return true
		end
	end
	return false
end

function YayMounts:EventHandler(arg1)
	if  arg1 == "PLAYER_REGEN_DISABLED" then
		playerCombat = true
		if class == "SHAMAN" then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645))
		elseif class == "DRUID" then
			if not self.db.char.settings["TTT"] then
				YayMounts:SetMacroButton("[swimming] "..YayMounts:SpellToName(1066).."; "..YayMounts:SpellToName(783))
			else
				YayMounts:SetMacroButton("[swimming] !"..YayMounts:SpellToName(1066).."; !"..YayMounts:SpellToName(783))
			end
		elseif class == "HUNTER" then
			YayMounts:Hunters()
		elseif class == "MAGE" or class == "PRIEST" then
			YayMounts:PriestOrMage()
		end
	elseif arg1 == "PLAYER_REGEN_ENABLED" then
		playerCombat = false
		YayMounts:SetMacroButton()
	end
end
--------------------------------------------------------------
--Keybinding stuff
--------------------------------------------------------------
function YayMounts:Keybinding(key, buttonID)
	if not InCombatLockdown() then
		if buttonID == 1 then
			SetBinding(key, "YAYMOUNTBINDING")
			yayMountsBindingButton2:SetKey(GetBindingKey("YAYMOUNTBINDING2"))
		elseif buttonID == 2 then
			SetBinding(key, "YAYMOUNTBINDING2")
			yayMountsBindingButton:SetKey(GetBindingKey("YAYMOUNTBINDING"))
		end
		SaveBindings(GetCurrentBindingSet())
	end
end